/*! \file boolean_analyzer.h
    This file defines the interface of the boolean expression analyzer.
 */

#ifndef BOOLEAN_ANALYSER_HEADER

#include "grammar_types.h"
#include "linked_list.h"

#define MAX_ERROR 2048

#ifndef BOOLEAN_ANALIZER

/*! \brief If the parser finds an sintaxic error, then the array boolean_parsing_error contains a zero
           terminated sequence of characters that describes the error.
 */

extern char boolean_parsing_error[MAX_ERROR];

/*! \brief The boolean analyser itself.
    \param src Pointer to a zero terminated string of characters that represnts the boolean expression.
    \param l Pointer to a linked list that will be used to store the RPN representation of the boolean
           expression.
    \return Upon successfull completion, the function returns the value 0. Otherwise, the function returns
            the value 1. If the returned value is 1, then the array boolean_parsing_error contains a description
            of the error.
 */

int boolean_spliter(char *src, List *l);

/*! \example test_bool.c
    This simple test program shows how to use the analyser of boolean expressions.
 */

#endif

#define BOOLEAN_ANALYSER_HEADER
#endif


