/* ----------------------------------------------------- */
/* This program illustrates the use of the boolean ex- */
/* -pression analyzer. */
/* ----------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include "boolean_analyzer.h"

/* ----------------------------------------------------- */
/* This is the in-memory grammatical parser */
/* ----------------------------------------------------- */

#define NUMBER_OF_TEST 5

char *expr[] = {
                 "b1 OR b2 AND b3 OR (b4 AND b5) XOR NOT b6",
                 "b1 OR b2 AND b3 OR (b4 AND b5)",
                 "b1 OR b2 AND",
                 "b1 OR b2 AND b3 OR ((b4 AND b5) OR b1)", "(b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1)) AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) XOR NOT test AND b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) OR NOT b1 OR b2 AND b3 OR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) XOR ((b4 AND b5) OR b1) AND NOT toto OR gros_test_de_la_mort XOR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 XOR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 XOR ((b4 AND b5) OR b1) AND b1 OR b2 AND b3 AND ((b4 AND b5) OR b1) AND b1 OR b2 AND b3" };

int main (int argc, char *argv[])
{
  int res, i;
  List list;
  ListElmt *element;

  for (i=0; i<NUMBER_OF_TEST; i++)
  {

    /* --------------------------------------------------- */
    /* Initialize the linked list */
    /* --------------------------------------------------- */

    init_lined_list (&list);

    /* --------------------------------------------------- */
    /* Call the boolean expression analyzer */
    /* --------------------------------------------------- */

    fprintf (stdout, "\n--> %s:\n", expr[i]);
    fflush (stdout);

    res = boolean_spliter(expr[i], &list);

    if (res == 1)
    {
      fprintf (stdout, "\nParsing error: %s\n", boolean_parsing_error);
      fflush (stdout);
      destroy_linked_list (&list);
      continue;
    }

    /* --------------------------------------------------- */
    /* Now print the RPN conversion */
    /* --------------------------------------------------- */

    element = list_head (&list);

    while (element != NULL)
    {
      switch (GET_TYPE(element))
      {
        case word: { fprintf (stdout, "\n%s", GET_WORD(element)); }; break;
        case operator: {
                         switch (GET_OPERATOR(element))
                         {
                           case or: { fprintf (stdout, "\nor()"); }; break;
                           case and: { fprintf (stdout, "\nand()"); }; break;
                           case xor: { fprintf (stdout, "\nxor()"); }; break;
                           case not: { fprintf (stdout, "\nnot()"); }; break;
                           case undef: { fprintf (stdout, "\nundef"); }; break;
                         }
                       }; break;
      }

      element = list_next(element);
    }

    fprintf (stdout, "\n");

    /* --------------------------------------------------- */
    /* Destroy the linked list */
    /* --------------------------------------------------- */

    destroy_linked_list (&list);
    fflush (stdout);
  }

  return 0;
}
